@@ -8,7 +8,7 @@ import logging
 from typing import Callable, List, Optional
 
 from iotas.backup_manager import BackupManager
-from iotas import const
+import iotas.const as const
 import iotas.config_manager
 from iotas.database import Database
 from iotas.migration_assistant import MigrationAssistant
@@ -50,18 +50,16 @@ class Application(Adw.Application):
         self.__high_contrast_css_resource = "{}/ui/high_contrast_style.css".format(
             self.props.resource_base_path
         )
-        self.__base_css_provider = None
-        self.__light_css_provider = None
-        self.__dark_css_provider = None
-        self.__index_category_label_css_provider = None
+        self.__base_css_provider: Gtk.CssProvider = None
+        self.__light_css_provider: Gtk.CssProvider = None
+        self.__dark_css_provider: Gtk.CssProvider = None
+        self.__index_category_label_css_provider: Gtk.CssProvider = None
         style_manager = Adw.StyleManager.get_default()
         style_manager.connect("notify::dark", lambda _o, _v: self.__apply_css())
         style_manager.connect("notify::high-contrast", lambda _o, _v: self.__apply_css())
 
-        self.cursors = {}
-        self.__actions = {}
-        self.__backup_manager = None
-        self.__preferences_dialog = None
+        self.__actions: dict[str, Gio.SimpleAction] = {}
+        self.__preferences_dialog: Optional[PreferencesDialog] = None
         self.__previous_version = ""
 
         self.connect("startup", lambda _o: self.__on_startup())
