@@ -7,6 +7,7 @@ from gi.repository import Adw, Gdk, Gio, GLib, GObject, Gtk, GtkSource, Pango
 
 import logging
 from typing import Any, Optional
+import webbrowser
 
 from iotas.category_header_bar import CategoryHeaderBar
 from iotas.category_treeview_list_store import CategoryTreeViewListStore
@@ -17,6 +18,11 @@ from iotas.export_dialog import ExportDialog
 from iotas.focus_mode_helper import FocusModeHelper
 from iotas.font_size_selector import FontSizeSelector
 from iotas.note import Note
+from iotas.text_utils import (
+    parse_any_url_at_iter,
+    parse_markdown_automatic_link,
+    parse_markdown_inline_link,
+)
 from iotas.theme_selector import ThemeSelector
 from iotas.ui_utils import add_mouse_button_accel, is_likely_mobile_device
 
@@ -32,11 +38,14 @@ class Editor(Adw.Bin):
     }
 
     LINE_LENGTH_STEP = 50
-    SETTING_CHANGE_NOTIFICATION_DURATION = 3.0
+    TOAST_DURATION = 3.0
     FONT_SETTING_KEYS = ["monospace-font-name", "document-font-name"]
     HEADERBAR_HIDE_DELAY = 2000
     MARGIN_BELOW_CURSOR = 22
 
+    _top_bar_revealer = Gtk.Template.Child()
+    _bottom_bar_revealer = Gtk.Template.Child()
+
     _title_label = Gtk.Template.Child()
     _is_dirty = Gtk.Template.Child()
     _headerbar_stack = Gtk.Template.Child()
@@ -48,16 +57,17 @@ class Editor(Adw.Bin):
     _render_search_header_bar = Gtk.Template.Child()
 
     _menu_button = Gtk.Template.Child()
+    _title_button = Gtk.Template.Child()
+    _content_overlay = Gtk.Template.Child()
     _render_edit_stack = Gtk.Template.Child()
     _render_edit_button_stack = Gtk.Template.Child()
     _render_button = Gtk.Template.Child()
     _edit_button = Gtk.Template.Child()
     _sourceview = Gtk.Template.Child()
     _editor_scrolledwin = Gtk.Template.Child()
+    _read_only_image = Gtk.Template.Child()
     _render_loading = Gtk.Template.Child()
-    _read_only_banner = Gtk.Template.Child()
-    _timed_notification = Gtk.Template.Child()
-    _toolbar_view = Gtk.Template.Child()
+    _toast_overlay = Gtk.Template.Child()
     _top_margin_box = Gtk.Template.Child()
     _bottom_margin_box = Gtk.Template.Child()
 
@@ -70,6 +80,7 @@ class Editor(Adw.Bin):
         self.__sync_authenticated = False
         self.__buffer_handlers = []
         self.__current_note_handlers = []
+        self.__queued_toasts = []
         self.__render_view = None
         self.__font_families = {}
         self.__html_generator = None
@@ -101,6 +112,10 @@ class Editor(Adw.Bin):
         self._rename_header_bar.connect("renamed", self.__on_title_renamed)
         self._rename_header_bar.connect("cancelled", self.__on_rename_cancelled)
 
+        self._formatting_header_bar.connect(
+            "height-change", lambda _o: self.__update_toolbar_underlay_margins()
+        )
+
         self._render_edit_button_stack.set_visible(
             iotas.config_manager.get_markdown_render_enabled()
         )
@@ -109,6 +124,9 @@ class Editor(Adw.Bin):
 
         style_manager = Adw.StyleManager.get_default()
         style_manager.connect("notify::dark", lambda _o, _v: self.__update_scheme_and_dark_style())
+        style_manager.connect(
+            "notify::high-contrast", lambda _o, _v: self.__update_scheme_and_dark_style()
+        )
 
         iotas.config_manager.settings.connect(
             f"changed::{iotas.config_manager.MARKDOWN_DETECT_SYNTAX}",
@@ -132,7 +150,7 @@ class Editor(Adw.Bin):
         )
         iotas.config_manager.settings.connect(
             f"changed::{iotas.config_manager.LINE_LENGTH}",
-            lambda _o, _k: self.__on_line_length_changed(),
+            lambda _o, _k: self.__refresh_line_length_from_setting(),
         )
 
         # Use system setting for monospace font family
@@ -189,6 +207,7 @@ class Editor(Adw.Bin):
         self._formatting_header_bar.setup(self._sourceview)
         window = self.get_root()
         window.connect("notify::fullscreened", self.__on_window_fullscreened)
+        self._sourceview.setup()
 
     def init_note(self, note: Note) -> None:
         """Initialise editing session.
@@ -219,7 +238,7 @@ class Editor(Adw.Bin):
         self.__update_title()
         self.__update_dirty()
 
-        self._read_only_banner.set_revealed(note.read_only)
+        self._read_only_image.set_visible(note.read_only)
         self._sourceview.set_editable(not note.read_only)
         self.__update_readonly_actions()
 
@@ -232,7 +251,6 @@ class Editor(Adw.Bin):
         ):
             self._headerbar_stack.set_visible_child(self._main_header_bar)
 
-        self._sourceview.set_visible(True)
         if self._render_edit_stack.get_visible_child() == self.__render_view:
             self.__toggle_render()
         else:
@@ -247,15 +265,16 @@ class Editor(Adw.Bin):
 
         self.__update_scheme_and_dark_style()
 
-        self._toolbar_view.set_reveal_top_bars(True)
+        self._top_bar_revealer.set_reveal_child(True)
         if (
             iotas.config_manager.get_markdown_detect_syntax()
             and self.__formatting_bar_visibility_setting != HeaderBarVisibility.DISABLED
         ):
-            self._toolbar_view.set_reveal_bottom_bars(True)
-        elif self._toolbar_view.get_reveal_bottom_bars():
-            self._toolbar_view.set_reveal_bottom_bars(False)
+            self._bottom_bar_revealer.set_reveal_child(True)
+        elif self._bottom_bar_revealer.get_reveal_child():
+            self._bottom_bar_revealer.set_reveal_child(False)
         self.__update_margin_box_heights()
+        GLib.idle_add(self.__update_toolbar_underlay_margins)
 
         self.__check_and_hide_bars_after_delay()
 
@@ -294,6 +313,7 @@ class Editor(Adw.Bin):
         self.__current_note_handlers = []
 
         self.__focus_mode_helper.active = False
+        self._bottom_bar_revealer.set_visible(True)
 
         self.current_note = None
 
@@ -341,7 +361,7 @@ class Editor(Adw.Bin):
 
         # Don't focus if search etc toolbar is active
         if (
-            self._toolbar_view.get_reveal_top_bars()
+            self._top_bar_revealer.get_reveal_child()
             and self._headerbar_stack.get_visible_child() != self._main_header_bar
         ):
             return
@@ -405,6 +425,14 @@ class Editor(Adw.Bin):
         if not self.__click_in_bottom_margin(gesture):
             self.__check_and_hide_formatting_bar()
 
+        event = gesture.get_current_event()
+        if event is None:
+            return
+        state = event.get_modifier_state()
+        if state & Gdk.ModifierType.CONTROL_MASK and state & Gdk.ModifierType.BUTTON1_MASK:
+            _interpreted, x, y = gesture.get_point(None)
+            self.__check_and_handle_link_click(x, y)
+
     @Gtk.Template.Callback()
     def _on_top_margin_motion(
         self, _controller: Gtk.EventControllerMotion, _x: float, _y: float
@@ -483,7 +511,7 @@ class Editor(Adw.Bin):
         action.connect("activate", lambda _o, _p: self.cancel())
         action_group.add_action(action)
         self.__cancel_action = action
-        app.set_accels_for_action("editor.go-back-or-cancel", ["Escape", "<Alt>Left"])
+        app.set_accels_for_action("editor.go-back-or-cancel", ["Escape"])
 
         action = Gio.SimpleAction.new("clear-category")
         action.connect("activate", lambda _o, _p: self.__on_clear_category())
@@ -520,19 +548,19 @@ class Editor(Adw.Bin):
         app.set_accels_for_action("editor.toggle-line-length-limit", ["<Shift><Control>l"])
 
         action = Gio.SimpleAction.new("increase-font-size")
-        action.connect("activate", lambda _o, _v: self.__on_increase_font_size())
+        action.connect("activate", lambda _o, _v: self.__font_size_selector.increase())
         action_group.add_action(action)
         app.set_accels_for_action("editor.increase-font-size", ["<Control>plus", "<Control>equal"])
 
         action = Gio.SimpleAction.new("decrease-font-size")
-        action.connect("activate", lambda _o, _v: self.__on_decrease_font_size())
+        action.connect("activate", lambda _o, _v: self.__font_size_selector.decrease())
         action_group.add_action(action)
         app.set_accels_for_action(
             "editor.decrease-font-size", ["<Control>minus", "<Control>underscore"]
         )
 
         action = Gio.SimpleAction.new("reset-font-size")
-        action.connect("activate", lambda _o, _v: self.__on_reset_font_size())
+        action.connect("activate", lambda _o, _v: self.__font_size_selector.reset())
         action_group.add_action(action)
         app.set_accels_for_action("editor.reset-font-size", ["<Control>0"])
 
@@ -547,6 +575,21 @@ class Editor(Adw.Bin):
         action_group.add_action(action)
         app.set_accels_for_action("editor.show-menu", ["F10"])
 
+        action = Gio.SimpleAction.new("focus-editor")
+        action.connect("activate", lambda _o, _p: self.__focus_editor())
+        action_group.add_action(action)
+        app.set_accels_for_action("editor.focus-editor", ["<Alt>e"])
+
+        action = Gio.SimpleAction.new("focus-header-bar")
+        action.connect("activate", lambda _o, _p: self.__focus_header_bar())
+        action_group.add_action(action)
+        app.set_accels_for_action("editor.focus-header-bar", ["<Alt>h"])
+
+        action = Gio.SimpleAction.new("focus-formatting-bar")
+        action.connect("activate", lambda _o, _p: self.__focus_formatting_bar())
+        action_group.add_action(action)
+        app.set_accels_for_action("editor.focus-formatting-bar", ["<Alt>f"])
+
         app.get_active_window().insert_action_group("font-size-selector", action_group)
         self.__action_group = action_group
 
@@ -643,7 +686,7 @@ class Editor(Adw.Bin):
         self._sourceview.grab_focus()
 
     def __on_clear_category(self) -> None:
-        if not self._toolbar_view.get_reveal_top_bars():
+        if not self._top_bar_revealer.get_reveal_child():
             return
         if self._headerbar_stack.get_visible_child() == self._category_header_bar:
             self._category_header_bar.clear_and_apply()
@@ -697,27 +740,20 @@ class Editor(Adw.Bin):
         if self.active and self._render_edit_stack.get_visible_child() == self.__render_view:
             self.__render_view.render_retaining_scroll(self.__note, None)
 
-    def __on_line_length_changed(self) -> None:
-        length = iotas.config_manager.get_line_length()
-        logging.debug(f"Line length now {length}px")
-        self.__refresh_line_length_from_setting()
-
     def __on_increase_line_length(self) -> None:
         setting_max = iotas.config_manager.get_line_length_max()
         current_length = iotas.config_manager.get_line_length()
         if current_length == setting_max:
-            # Translators: Description, notification
-            self.__notify_setting_change(_("Line length limit already disabled"))
+            logging.info("Line length limit already disabled")
             return
         else:
             new_length = current_length + self.LINE_LENGTH_STEP
-        if new_length < self.get_width() - 2 * self._sourceview.BASE_MARGIN:
+        if new_length < self.get_width() - 2 * self._sourceview.STANDARD_MARGIN:
             iotas.config_manager.set_line_length(new_length)
-            self.__notify_line_length_change(new_length)
+            logging.info(f"Line length increased to {new_length}px")
         else:
             iotas.config_manager.set_line_length(setting_max)
-            # Translators: Description, notification
-            self.__notify_setting_change(_("Line length limit disabled"))
+            self.__notify_line_length_limit_disabled()
 
     def __on_decrease_line_length(self) -> None:
         setting_max = iotas.config_manager.get_line_length_max()
@@ -726,19 +762,25 @@ class Editor(Adw.Bin):
         if current_length == setting_max:
             new_length = min(self.get_width(), iotas.config_manager.get_default_line_length())
             iotas.config_manager.set_line_length(new_length)
-            self.__notify_line_length_change(new_length)
+            self.__notify_line_length_limit_enabled(new_length)
         elif new_length >= self.LINE_LENGTH_STEP:
             iotas.config_manager.set_line_length(new_length)
-            self.__notify_line_length_change(new_length)
+            logging.info(f"Line length decreased to {new_length}px")
 
     def __on_line_length_limit_toggle(self) -> None:
         setting_maximum = iotas.config_manager.get_line_length_max()
         if iotas.config_manager.get_line_length() == setting_maximum:
-            iotas.config_manager.set_line_length(iotas.config_manager.get_default_line_length())
+            new_length = iotas.config_manager.get_default_line_length()
+            iotas.config_manager.set_line_length(new_length)
+            self.__notify_line_length_limit_enabled(new_length)
         else:
             iotas.config_manager.set_line_length(setting_maximum)
+            self.__notify_line_length_limit_disabled()
 
     def __on_window_fullscreened(self, window: Gtk.Window, _param: GObject.ParamSpec) -> None:
+        if not self.active:
+            return
+
         if self.__header_bar_visibility_setting == HeaderBarVisibility.AUTO_HIDE_FULLSCREEN_ONLY:
             if window.is_fullscreen():
                 self.__hide_header_bar()
@@ -753,6 +795,8 @@ class Editor(Adw.Bin):
                 else:
                     self.__show_formatting_bar()
 
+        self.__update_toolbar_underlay_margins()
+
     def __on_render_single_click(self) -> None:
         self.__check_and_hide_header_bar()
 
@@ -787,18 +831,6 @@ class Editor(Adw.Bin):
         self.__check_and_hide_bars_after_delay()
         self._sourceview.grab_focus()
 
-    def __on_increase_font_size(self) -> None:
-        if self.__font_size_selector.increase():
-            self.__notify_font_size()
-
-    def __on_decrease_font_size(self) -> None:
-        if self.__font_size_selector.decrease():
-            self.__notify_font_size()
-
-    def __on_reset_font_size(self) -> None:
-        self.__font_size_selector.reset()
-        self.__notify_font_size()
-
     def __on_focus_mode_toggle(self, action: Gio.SimpleAction, state: bool) -> None:
         action.set_state(state)
         self.__focus_mode_helper.active = state
@@ -845,22 +877,41 @@ class Editor(Adw.Bin):
             scroll_to = lowest_visible + self.MARGIN_BELOW_CURSOR - adj.get_page_size()
             adj.set_value(scroll_to)
 
-    def __notify_line_length_change(self, length: int) -> None:
+    def __notify_line_length_limit_enabled(self, new_length: int) -> None:
+        logging.info(f"Line length re-enabled, to {new_length}px")
         # Translators: Description, notification, {0} is a number
-        msg = _("Line length now {0}px").format(length)
-        self.__notify_setting_change(msg)
+        msg = _("Line length now {0}px").format(new_length)
+        self.__show_toast(msg)
 
-    def __notify_font_size(self) -> None:
-        size = iotas.config_manager.get_font_size()
-        # Translators: Description, notification, {0} is a number
-        msg = _("Font size now {0}pt").format(size)
-        self.__notify_setting_change(msg)
+    def __notify_line_length_limit_disabled(self) -> None:
+        logging.info("Line length limit disabled")
+        # Translators: Description, notification
+        self.__show_toast(_("Line length limit disabled"))
 
-    def __notify_setting_change(self, msg: str) -> None:
-        self._timed_notification.show(
-            msg,
-            self.SETTING_CHANGE_NOTIFICATION_DURATION,
-        )
+    def __show_toast(self, msg: str) -> None:
+        toast = Adw.Toast.new(msg)
+        toast.set_priority(Adw.ToastPriority.HIGH)
+        toast.set_timeout(self.TOAST_DURATION)
+        toast.connect("dismissed", self.__untrack_dismissed_toast)
+
+        # TODO change to ToastOverlay.dismiss_all with adwaita 1.7+
+        # self._toast_overlay.dismiss_all()
+        self.__dismiss_any_toast()
+
+        self._toast_overlay.add_toast(toast)
+        self.__queued_toasts.append(toast)
+
+    # Ensure that when eg. bumping through font sizes the toast for the previous size doesn't get
+    # shown after the final size. Upcoming adwaita will provide cleaner method via dismiss_all.
+    def __dismiss_any_toast(self) -> None:
+        for toast in self.__queued_toasts:
+            toast.dismiss()
+
+    def __untrack_dismissed_toast(self, toast: Adw.Toast) -> None:
+        try:
+            self.__queued_toasts.remove(toast)
+        except ValueError:
+            pass
 
     def __init_webkit(self) -> None:
         # Lazy import module in an attempt to reduce startup performance hit on devices with
@@ -961,7 +1012,7 @@ class Editor(Adw.Bin):
 
     def __edit_category(self) -> None:
         """Show the category editing."""
-        if not self._toolbar_view.get_reveal_top_bars():
+        if not self._top_bar_revealer.get_reveal_child():
             self.__show_header_bar()
         if self._headerbar_stack.get_visible_child() == self._category_header_bar:
             return
@@ -976,7 +1027,7 @@ class Editor(Adw.Bin):
         """Show the title rename entry."""
         if self.__note.read_only:
             return
-        if not self._toolbar_view.get_reveal_top_bars():
+        if not self._top_bar_revealer.get_reveal_child():
             self.__show_header_bar()
         if self._headerbar_stack.get_visible_child() == self._rename_header_bar:
             return
@@ -996,7 +1047,7 @@ class Editor(Adw.Bin):
             self._render_search_header_bar.enter(self.__render_view, resuming, type_to_search=False)
             self._headerbar_stack.set_visible_child(self._render_search_header_bar)
 
-        if not self._toolbar_view.get_reveal_top_bars():
+        if not self._top_bar_revealer.get_reveal_child():
             self.__show_header_bar()
 
     def __exit_search(self) -> None:
@@ -1021,10 +1072,8 @@ class Editor(Adw.Bin):
             if self._headerbar_stack.get_visible_child() == self._search_header_bar:
                 self.__exit_search()
 
-            # Don't use the ToolbarView transition as this results in visual glitch as the WebView
-            # tries to render into the expanding view
-            if self._toolbar_view.get_reveal_bottom_bars():
-                self._formatting_header_bar.set_visible(False)
+            if self._bottom_bar_revealer.get_reveal_child():
+                self._bottom_bar_revealer.set_visible(False)
 
             if self.__render_view is not None:
                 self.__continue_loading_webkit()
@@ -1042,8 +1091,8 @@ class Editor(Adw.Bin):
             self._sourceview.set_visible(True)
             self._render_edit_stack.set_visible_child(self._editor_scrolledwin)
             self.__check_and_terminate_webkit_process()
-            if not self._formatting_header_bar.get_visible():
-                self._formatting_header_bar.set_visible(True)
+            self._bottom_bar_revealer.set_visible(True)
+            self.__update_formatting_bar_visibility()
             self.__check_and_hide_bars_after_delay()
             self.focus_textview_if_editing()
             self._render_search_header_bar.disable_actions()
@@ -1074,8 +1123,18 @@ class Editor(Adw.Bin):
         style_scheme_manager = GtkSource.StyleSchemeManager.get_default()
 
         scheme_id = iotas.config_manager.get_editor_theme()
-        scheme_id = f"{scheme_id}-dark" if style_manager.get_dark() else scheme_id
-        scheme = style_scheme_manager.get_scheme(scheme_id)
+        if style_manager.get_dark():
+            scheme_id = f"{scheme_id}-dark"
+
+        scheme = None
+        if style_manager.get_high_contrast():
+            high_contrast_id = f"{scheme_id}-high-contrast"
+            scheme = style_scheme_manager.get_scheme(high_contrast_id)
+            if not scheme:
+                logging.debug(f"Scheme '{high_contrast_id}' doesn't exist")
+
+        if scheme is None:
+            scheme = style_scheme_manager.get_scheme(scheme_id)
         buffer.set_style_scheme(scheme)
 
     def __refresh_line_length_from_setting(self) -> None:
@@ -1154,7 +1213,7 @@ class Editor(Adw.Bin):
 
     def __show_header_bar(self) -> None:
         """Show the headerbar."""
-        self._toolbar_view.set_reveal_top_bars(True)
+        self._top_bar_revealer.set_reveal_child(True)
         self.__update_margin_box_heights()
 
     def __check_and_hide_bars_after_delay(self) -> None:
@@ -1176,8 +1235,8 @@ class Editor(Adw.Bin):
 
     def __both_toolbars_hidden(self) -> bool:
         return (
-            not self._toolbar_view.get_reveal_top_bars()
-            and not self._toolbar_view.get_reveal_bottom_bars()
+            not self._top_bar_revealer.get_reveal_child()
+            and not self._bottom_bar_revealer.get_reveal_child()
         )
 
     def __neither_toolbar_auto_hiding(self) -> bool:
@@ -1193,23 +1252,23 @@ class Editor(Adw.Bin):
         elif self._menu_button.get_popover().get_visible():
             return
 
-        if self._toolbar_view.get_reveal_top_bars():
-            self._toolbar_view.set_reveal_top_bars(False)
+        if self._top_bar_revealer.get_reveal_child():
+            self._top_bar_revealer.set_reveal_child(False)
             self.focus_textview_if_editing()
 
         self.__update_margin_box_heights()
 
     def __show_formatting_bar(self) -> None:
-        self._toolbar_view.set_reveal_bottom_bars(True)
+        self._bottom_bar_revealer.set_reveal_child(True)
         self.__update_margin_box_heights()
 
     def __hide_formatting_bar(self) -> None:
-        self._toolbar_view.set_reveal_bottom_bars(False)
+        self._bottom_bar_revealer.set_reveal_child(False)
         self.focus_textview_if_editing()
         self.__update_margin_box_heights()
 
     def __check_and_hide_formatting_bar(self) -> bool:
-        if not self._toolbar_view.get_reveal_bottom_bars():
+        if not self._bottom_bar_revealer.get_reveal_child():
             return False
 
         if not iotas.config_manager.get_markdown_detect_syntax():
@@ -1230,7 +1289,7 @@ class Editor(Adw.Bin):
         if self.__hiding_bars_timeout:
             GLib.source_remove(self.__hiding_bars_timeout)
             self.__hiding_bars_timeout = None
-        if self._toolbar_view.get_reveal_bottom_bars():
+        if self._bottom_bar_revealer.get_reveal_child():
             return
         if not self.active or self._render_edit_stack.get_visible_child() == self.__render_view:
             return
@@ -1240,7 +1299,7 @@ class Editor(Adw.Bin):
             self.__show_formatting_bar()
 
     def __check_and_hide_header_bar(self) -> None:
-        if not self._toolbar_view.get_reveal_top_bars():
+        if not self._top_bar_revealer.get_reveal_child():
             return
 
         hide = False
@@ -1256,12 +1315,30 @@ class Editor(Adw.Bin):
         if self.__hiding_bars_timeout:
             GLib.source_remove(self.__hiding_bars_timeout)
             self.__hiding_bars_timeout = None
-        if not self._toolbar_view.get_reveal_top_bars():
+        if not self._top_bar_revealer.get_reveal_child():
             self.__show_header_bar()
 
     def __update_for_formatting_bar_setting_change(self) -> None:
         self.__cache_formatting_bar_visibility()
+        if self.active:
+            self.__update_toolbar_underlay_margins()
+            self.__update_formatting_bar_visibility()
 
+    def __update_toolbar_underlay_margins(self) -> None:
+        pad = iotas.config_manager.get_editor_header_bar_visible_for_window_state()
+        margin = const.HEADER_BAR_HEIGHT if pad else 0
+        if self._content_overlay.get_margin_top() != margin:
+            self._content_overlay.set_margin_top(margin)
+
+        pad = iotas.config_manager.get_editor_formatting_bar_visible_for_window_state()
+        if pad:
+            margin = self._formatting_header_bar.get_property("height-request")
+        else:
+            margin = 0
+        if self._editor_scrolledwin.get_margin_bottom() != margin:
+            self._editor_scrolledwin.set_margin_bottom(margin)
+
+    def __update_formatting_bar_visibility(self) -> None:
         if not self.active or self._render_edit_stack.get_visible_child() == self.__render_view:
             return
 
@@ -1269,7 +1346,7 @@ class Editor(Adw.Bin):
             return
 
         setting = self.__formatting_bar_visibility_setting
-        if self._toolbar_view.get_reveal_bottom_bars():
+        if self._bottom_bar_revealer.get_reveal_child():
             self.__check_and_hide_formatting_bar()
         else:
             show = False
@@ -1287,7 +1364,7 @@ class Editor(Adw.Bin):
             return
 
         setting = self.__header_bar_visibility_setting
-        if self._toolbar_view.get_reveal_top_bars():
+        if self._top_bar_revealer.get_reveal_child():
             self.__check_and_hide_header_bar()
         else:
             show = False
@@ -1297,6 +1374,7 @@ class Editor(Adw.Bin):
                 show = not self.get_root().is_fullscreen()
             if show:
                 self.__show_header_bar()
+        self.__update_toolbar_underlay_margins()
 
     def __cache_header_bar_visibility(self) -> None:
         self.__header_bar_visibility_setting = HeaderBarVisibility(
@@ -1329,17 +1407,17 @@ class Editor(Adw.Bin):
         margin = self._sourceview.get_property("top-margin")
         # When the bar is revealed the margin area is shrunk to allow interaction with the top of
         # the buffer
-        height = 1 if self._toolbar_view.get_reveal_top_bars() else margin
+        height = 1 if self._top_bar_revealer.get_reveal_child() else margin
         self._top_margin_box.set_property("height-request", height)
 
-        height = 1 if self._toolbar_view.get_reveal_bottom_bars() else margin
+        height = 1 if self._bottom_bar_revealer.get_reveal_child() else margin
         self._bottom_margin_box.set_property("height-request", height)
 
     def __click_in_top_margin(self, gesture: Gtk.Gesture) -> bool:
         (interpreted, _, y) = gesture.get_point(None)
         if not interpreted:
             return False
-        if not self._toolbar_view.get_reveal_top_bars():
+        if not self._top_bar_revealer.get_reveal_child():
             return False
 
         return y < self._sourceview.get_property("top-margin")
@@ -1348,7 +1426,69 @@ class Editor(Adw.Bin):
         (interpreted, _, y) = gesture.get_point(None)
         if not interpreted:
             return False
-        if not self._toolbar_view.get_reveal_bottom_bars():
+        if not self._bottom_bar_revealer.get_reveal_child():
             return False
 
         return y > self._sourceview.get_height() - self._sourceview.get_property("top-margin")
+
+    def __focus_editor(self) -> None:
+        # Don't focus if showing render view
+        if self._render_edit_stack.get_visible_child() != self._editor_scrolledwin:
+            return
+
+        self._sourceview.grab_focus()
+
+    def __focus_header_bar(self) -> None:
+        self.__show_header_bar()
+        if self._headerbar_stack.get_visible_child() == self._rename_header_bar:
+            self._rename_header_bar.focus()
+        elif self._headerbar_stack.get_visible_child() == self._search_header_bar:
+            self._search_header_bar.refocus_search_and_select()
+        elif self._headerbar_stack.get_visible_child() == self._category_header_bar:
+            self._category_header_bar.focus()
+        elif self._headerbar_stack.get_visible_child() == self._render_search_header_bar:
+            self._render_search_header_bar.focus()
+        else:
+            self._title_button.grab_focus()
+
+    def __focus_formatting_bar(self) -> None:
+        if not iotas.config_manager.get_markdown_detect_syntax():
+            return
+        if self.__formatting_bar_visibility_setting == HeaderBarVisibility.DISABLED:
+            return
+        self.__show_formatting_bar()
+        self._formatting_header_bar.focus()
+
+    def __check_and_handle_link_click(self, x: float, y: float) -> None:
+        link = None
+
+        buffer_x, buffer_y = self._sourceview.window_to_buffer_coords(
+            Gtk.TextWindowType.TEXT,
+            x,
+            y,
+        )
+        over_text, iter = self._sourceview.get_iter_at_location(buffer_x, buffer_y)
+        if not over_text:
+            logging.debug("Not over text for URL ctrl-click")
+            return
+
+        # Check and open basic link
+        result = parse_any_url_at_iter(iter, http_only=True)
+        if result:
+            link = result.link
+        else:
+            # Check for a markdown inline and automatic links
+            classes = iter.get_buffer().get_context_classes_at_iter(iter)
+            if "inline-link" in classes:
+                result = parse_markdown_inline_link(iter)
+                if result:
+                    link = result.link
+            elif "automatic-link" in classes:
+                result = parse_markdown_automatic_link(iter)
+                if result:
+                    link = result.link
+
+        if link:
+            logging.info(f"Opening URI from ctrl-click: {link}")
+            webbrowser.open(link)
+            self.__show_toast(_("Opening link in browser"))
