@@ -11,6 +11,7 @@ import re
 from typing import Callable, Optional
 
 import iotas.config_manager
+from iotas import const
 from iotas.ordered_list_utils import (
     check_line_for_ordered_list_item,
     calculate_ordered_list_index,
@@ -40,7 +41,7 @@ class EditorTextView(GtkSource.View):
 
     SOURCEVIEW_NO_SPELLCHECK_TAG = "gtksourceview:context-classes:no-spell-check"
 
-    BASE_MARGIN = 36
+    STANDARD_MARGIN = 36
     MINIMUM_MARGIN = 10
 
     def __init__(self) -> None:
@@ -57,6 +58,11 @@ class EditorTextView(GtkSource.View):
             f"changed::{iotas.config_manager.FONT_SIZE}", lambda _o, _k: self.__update_font()
         )
 
+        iotas.config_manager.settings.connect(
+            f"changed::{iotas.config_manager.EDITOR_HEADER_BAR_VISIBILTY}",
+            lambda _o, _k: self.__update_margins(self.get_width()),
+        )
+
         controller = Gtk.EventControllerKey()
         controller.connect("key-pressed", self.__on_key_pressed)
         self.add_controller(controller)
@@ -73,6 +79,13 @@ class EditorTextView(GtkSource.View):
         # TODO Temporary hack to provide access to spelling on mobile
         self.__setup_longpress_touch_menu()
 
+    def setup(self) -> None:
+        """Initialisation after window creation."""
+        window = self.get_root()
+        window.connect(
+            "notify::fullscreened", lambda _o, _v: self.__update_margins(self.get_width())
+        )
+
     def do_size_allocate(self, width: int, height: int, baseline: int) -> None:
         """Allocates widget with a transformation that translates the origin to the position in
         allocation.
@@ -131,7 +144,7 @@ class EditorTextView(GtkSource.View):
         # spelling disabled
         if self.__spellchecker is None or not self.__spellchecker.enabled:
             return
-        buffer_x, buffer_y = self.window_to_buffer_coords(2, int(x), int(y))
+        buffer_x, buffer_y = self.window_to_buffer_coords(Gtk.TextWindowType.TEXT, int(x), int(y))
         iter = self.get_iter_at_location(buffer_x, buffer_y)[1]
         self.__spellchecker.move_click_mark(iter)
         self.__spellchecker.populate_menu(self.__longpress_popover.get_menu_model())
@@ -172,24 +185,32 @@ class EditorTextView(GtkSource.View):
         min_width_for_y_margin = 400
         max_width_for_y_margin = self.__line_length if self.__line_length > 500 else 800
         if width <= min_width_for_y_margin:
-            y_margin = self.MINIMUM_MARGIN
+            y_margin_base = self.MINIMUM_MARGIN
         elif width < max_width_for_y_margin:
             value_range = max_width_for_y_margin - min_width_for_y_margin
-            y_range = float(self.BASE_MARGIN - self.MINIMUM_MARGIN)
+            y_range = float(self.STANDARD_MARGIN - self.MINIMUM_MARGIN)
             in_range = width - min_width_for_y_margin
-            y_margin = self.MINIMUM_MARGIN + int(in_range / value_range * y_range)
+            y_margin_base = self.MINIMUM_MARGIN + int(in_range / value_range * y_range)
         else:
-            y_margin = self.BASE_MARGIN
+            y_margin_base = self.STANDARD_MARGIN
+
+        # If the header bar is non hiding (set visible for the fullscreen state) there will be
+        # padding elsewhere to ensure the text avoids the headerbar. Otherwise we add it here.
+        if iotas.config_manager.get_editor_header_bar_visible_for_window_state():
+            y_margin = y_margin_base
+        else:
+            y_margin = y_margin_base + const.HEADER_BAR_HEIGHT
+
         if self.props.top_margin != y_margin:
             self.set_top_margin(y_margin)
 
         # Update side margins
         if width < 1:
             return
-        if self.__line_length > 0 and width - 2 * self.BASE_MARGIN > self.__line_length:
+        if self.__line_length > 0 and width - 2 * self.STANDARD_MARGIN > self.__line_length:
             x_margin = (width - self.__line_length) / 2
         else:
-            x_margin = y_margin
+            x_margin = y_margin_base
         if self.get_left_margin() != x_margin:
             self.set_left_margin(x_margin)
             self.set_right_margin(x_margin)
