@@ -33,7 +33,7 @@ class Index(Adw.BreakpointBin):
     NOTES_APP_MISSING_NOTIFICATION_DURATION = 8.0
     BREAKPOINT_WIDTH = 700
 
-    _split_view = Gtk.Template.Child()
+    _overlay_view = Gtk.Template.Child()
     _breakpoint = Gtk.Template.Child()
 
     _stack = Gtk.Template.Child()
@@ -41,6 +41,7 @@ class Index(Adw.BreakpointBin):
     _note_list_scroll = Gtk.Template.Child()
     _note_list = Gtk.Template.Child()
     _first_start = Gtk.Template.Child()
+    _loading = Gtk.Template.Child()
     _empty = Gtk.Template.Child()
     _search_info = Gtk.Template.Child()
     _search_empty = Gtk.Template.Child()
@@ -52,12 +53,12 @@ class Index(Adw.BreakpointBin):
     _selection_header_bar = Gtk.Template.Child()
     _title = Gtk.Template.Child()
 
-    _notification = Gtk.Template.Child()
-    _timed_notification = Gtk.Template.Child()
+    _toast_overlay = Gtk.Template.Child()
 
     _menu_button = Gtk.Template.Child()
     _new_button = Gtk.Template.Child()
 
+    _misc_banner = Gtk.Template.Child()
     _offline_banner = Gtk.Template.Child()
     _app_id_change_reauthenticate_banner = Gtk.Template.Child()
     _generic_reauthenticate_banner = Gtk.Template.Child()
@@ -77,6 +78,7 @@ class Index(Adw.BreakpointBin):
         self.__search_after_init = None
         self.__searching_before_selection = False
         self.__cancel_action = None
+        self.__syncing_toast = None
 
         self.__search_model = None
 
@@ -92,10 +94,6 @@ class Index(Adw.BreakpointBin):
             lambda _o, note: self._selection_header_bar.set_note_selected(note, False),
         )
 
-        if not iotas.config_manager.get_first_start():
-            # Translators: Description, notification
-            self._notification.show(_("Loading"), immediate=True)
-
         self._selection_header_bar.connect("categories-changed", self.__on_notes_categories_changed)
         self._selection_header_bar.connect("delete", self.__on_notes_deleted)
         self._selection_header_bar.connect("set-favourite", self.__on_notes_set_favourite)
@@ -119,12 +117,16 @@ class Index(Adw.BreakpointBin):
 
         iotas.config_manager.settings.connect(
             f"changed::{iotas.config_manager.PERSIST_SIDEBAR}",
-            lambda _o, _k: self.__update_for_sidebar_persistence_change(),
+            lambda _o, _k: self.__update_for_sidebar_pin_change(),
         )
 
         self._breakpoint.connect("apply", self.__on_breakpoint_apply)
         self._breakpoint.connect("unapply", self.__on_breakpoint_unapply)
 
+        self._overlay_view.connect(
+            "notify::show-sidebar", lambda _o, _v: self.__update_header_buttons_for_state()
+        )
+
     def setup(
         self,
         note_manager: NoteManager,
@@ -170,7 +172,7 @@ class Index(Adw.BreakpointBin):
             self._stack.set_visible_child(self._first_start)
             self._first_start.initialise_device()
         else:
-            self._stack.set_visible(False)
+            self._stack.set_visible_child(self._loading)
 
         self.__sync_manager.connect("ready", self.__on_sync_ready)
         self.__sync_manager.connect("started", self.__on_sync_started)
@@ -197,14 +199,15 @@ class Index(Adw.BreakpointBin):
         else:
             # Translators: Description, notification
             msg = _("Note deleted")
-        self._timed_notification.show(
-            msg,
-            self.UNDO_NOTE_DELETION_DURATION,
-            # Translators: Button
-            _("Undo"),
-            self.__undo_deletion,
-            self.__note_manager.on_deletion_undo_elapsed,
-        )
+
+        toast = Adw.Toast.new(msg)
+        toast.set_timeout(self.UNDO_NOTE_DELETION_DURATION)
+        # Translators: Button
+        toast.set_button_label(_("Undo"))
+        toast.connect("button-clicked", lambda _o: self.__undo_deletion())
+        toast.connect("dismissed", lambda _o: self.__note_manager.on_deletion_undo_elapsed())
+        self._toast_overlay.add_toast(toast)
+
         if self.__searching():
             self.__apply_search()
         else:
@@ -225,9 +228,7 @@ class Index(Adw.BreakpointBin):
         :param int width: Window width
         :param int height: Window height
         """
-        self.__update_for_sidebar_persistence_change(width)
-        if not iotas.config_manager.get_persist_sidebar():
-            self._split_view.set_collapsed(True)
+        self.__update_for_sidebar_pin_change(width)
         self._note_list.update_search_pagesize_for_height(height)
 
     def update_search_pagesize_for_height(self, height: int) -> None:
@@ -239,26 +240,20 @@ class Index(Adw.BreakpointBin):
 
     def show_note_conflict_alert(self) -> None:
         """Let the user know a sync conflict has occurred with the note being edited."""
-        self._timed_notification.show(
-            # Translators: Description, notification
-            _("Sync conflict with note being edited"),
-            self.SYNC_CONFLICT_NOTIFICATION_DURATION,
-        )
+        # Translators: Description, notification
+        toast = Adw.Toast.new(_("Sync conflict with note being edited"))
+        toast.set_timeout(self.SYNC_CONFLICT_NOTIFICATION_DURATION)
+        self._toast_overlay.add_toast(toast)
 
     def show_secret_service_failure_alert(self) -> None:
-        # Somewhat a clunky placeholder approach for now
-        self._timed_notification.show(
-            _(
-                # Translators: Description, notification. "Secret Service" and "gnome-keyring"
-                # should likely not be translated.
-                "Failure accessing Secret Service. Ensure you have a "
-                "provider like gnome-keyring which has a default keyring "
-                "setup that is unlocked."
-            ),
-            0.0,
-            # Translators: Button
-            _("OK"),
+        text = _(
+            # Translators: Description, notification. "Secret Service" and "gnome-keyring" should
+            # likely not be translated.
+            "Failure accessing Secret Service. Ensure you have a provider like gnome-keyring which "
+            "has a default keyring setup that is unlocked."
         )
+        self._misc_banner.set_title(text)
+        self._misc_banner.set_revealed(True)
 
     def refresh_after_note_closed(self, note: Note) -> None:
         """Refresh index after note has been closed.
@@ -301,6 +296,11 @@ class Index(Adw.BreakpointBin):
     def _on_reauthenticate(self, _button: Gtk.Button) -> None:
         self.emit("reauthenticate")
 
+    @Gtk.Template.Callback()
+    def _on_misc_banner_button(self, _button: Gtk.Button) -> None:
+        if self._misc_banner.get_revealed():
+            self._misc_banner.set_revealed(False)
+
     @Gtk.Template.Callback()
     def _on_edge_overshot(self, _obj: Gtk.ScrolledWindow, pos: Gtk.PositionType) -> None:
         if pos == Gtk.PositionType.BOTTOM:
@@ -309,7 +309,7 @@ class Index(Adw.BreakpointBin):
                     self._note_list.show_remaining_search_results()
             else:
                 if not self._note_list.older_notes_displayed:
-                    self.__load_older_notes(animate_in_notification=False)
+                    self.__load_older_notes()
 
     def __setup_actions(self) -> None:
         action_group = Gio.SimpleActionGroup.new()
@@ -340,11 +340,7 @@ class Index(Adw.BreakpointBin):
         app.set_accels_for_action("index.delete-from-index", ["<Shift>Delete"])
 
         action = Gio.SimpleAction.new("load-older-notes")
-        # Somehow the loading animation doesn't show if it comes from the action (in contrast to
-        # the edge scroll callback). So the notification is allowed to animate in for this case.
-        action.connect(
-            "activate", lambda _a, _p: self.__load_older_notes(animate_in_notification=True)
-        )
+        action.connect("activate", lambda _a, _p: self.__load_older_notes())
         action_group.add_action(action)
 
         action = Gio.SimpleAction.new("show-more")
@@ -464,10 +460,7 @@ class Index(Adw.BreakpointBin):
 
         self._note_list.refresh_section_visibility(self.__category)
         self._note_list.update_category_labels(self.__category)
-        if close_sidebar and (
-            not iotas.config_manager.get_persist_sidebar()
-            or self.get_width() < self.BREAKPOINT_WIDTH
-        ):
+        if close_sidebar:
             self.__close_sidebar()
         if self.get_root().using_keyboard_navigation:
             self._note_list.move_focus_to_list_top()
@@ -484,12 +477,21 @@ class Index(Adw.BreakpointBin):
 
     def __on_sync_started(self, _obj: GObject.Object) -> None:
         if self.active and iotas.config_manager.get_show_syncing_debug_notification():
+            # Another toast misuse replacing a revealer notification. Debug only at least.
+            # TODO in future look at replacing this with a (debug only) spinner.
+
             # Translators: Description, notification
-            self._notification.show(_("Syncing"))
+            self.__syncing_toast = Adw.Toast.new(_("Syncing"))
+
+            # A cludge, only passable for this debug functionality.
+            self.__syncing_toast.set_timeout(60.0)
+            self._toast_overlay.add_toast(self.__syncing_toast)
 
     def __on_sync_finished(self, _obj: SyncManager, changes: int) -> None:
         if self.active:
-            self._notification.hide()
+            if self.__syncing_toast:
+                self.__syncing_toast.dismiss()
+                self.__syncing_toast = None
             if self.__import_syncing:
                 self.__on_category_activated(self.__category)
                 self.__import_syncing = False
@@ -499,9 +501,9 @@ class Index(Adw.BreakpointBin):
 
                 # Translators: Description, notification, {} is a number
                 msg = ngettext("{} change", "{} changes", changes).format(changes)
-                self._timed_notification.show(
-                    msg, self.SYNC_UPDATE_NOTIFICATION_DURATION, show_icon=True
-                )
+                toast = Adw.Toast.new(msg)
+                toast.set_timeout(self.SYNC_UPDATE_NOTIFICATION_DURATION)
+                self._toast_overlay.add_toast(toast)
 
     def __on_missing_password(self, _obj: GObject.Object) -> None:
         app = Gio.Application.get_default()
@@ -513,11 +515,11 @@ class Index(Adw.BreakpointBin):
                 self._generic_reauthenticate_banner.set_revealed(True)
 
     def __on_missing_server_notes_capability(self, _obj: GObject.Object) -> None:
-        self._timed_notification.show(
-            # Translators: Description, notification
-            _("Sync failure. Is the Nextcloud Notes app installed on the server?"),
-            duration=self.NOTES_APP_MISSING_NOTIFICATION_DURATION,
-        )
+        # Translators: Description, notification
+        msg = _("Sync failure. Is the Nextcloud Notes app installed on the server?")
+        toast = Adw.Toast.new(msg)
+        toast.set_timeout(self.NOTES_APP_MISSING_NOTIFICATION_DURATION)
+        self._toast_overlay.add_toast(toast)
 
     def __on_index_key_pressed(
         self,
@@ -527,15 +529,15 @@ class Index(Adw.BreakpointBin):
         state: Gdk.ModifierType,
     ) -> bool:
         if keyval in (Gdk.KEY_Down, Gdk.KEY_KP_Down):
-            if (
-                not iotas.config_manager.get_persist_sidebar()
-                and not self._split_view.get_collapsed()
-            ):
-                # If we have an auto-expanded sidebar use the shortcut to send the focus
-                self.get_root().using_keyboard_navigation = True
-                self._sidebar.take_focus()
-            else:
+            if self._overlay_view.get_pin_sidebar():
                 self._note_list.move_focus_to_list_top()
+            else:
+                if self._overlay_view.get_show_sidebar():
+                    # If we have an auto-expanded sidebar use the shortcut to send the focus
+                    self.get_root().using_keyboard_navigation = True
+                    self._sidebar.take_focus()
+                else:
+                    self._note_list.move_focus_to_list_top()
             return Gdk.EVENT_STOP
         elif keyval in (Gdk.KEY_Up, Gdk.KEY_KP_Up):
             self._note_list.move_focus_to_list_top()
@@ -593,34 +595,38 @@ class Index(Adw.BreakpointBin):
             self.__toggle_sidebar()
 
     def __on_breakpoint_apply(self, _bp: Adw.Breakpoint) -> None:
-        self._split_view.set_collapsed(True)
+        if self._overlay_view.get_pin_sidebar():
+            self._overlay_view.set_pin_sidebar(False)
+            self._overlay_view.set_collapsed(True)
+
+        self._overlay_view.set_show_sidebar(False)
         self.__update_header_buttons_for_state()
 
     def __on_breakpoint_unapply(self, _bp: Adw.Breakpoint) -> None:
-        if iotas.config_manager.get_persist_sidebar():
-            self._split_view.set_collapsed(False)
-        else:
-            self._split_view.set_collapsed(True)
+        if iotas.config_manager.get_pin_sidebar():
+            self._overlay_view.set_pin_sidebar(True)
+            self._overlay_view.set_collapsed(False)
+            self._overlay_view.set_show_sidebar(True)
+
         self.__update_header_buttons_for_state()
 
-    def __load_older_notes(self, animate_in_notification: bool) -> None:
+    def __load_older_notes(self) -> None:
+        # Somewhat clunky misuse of toast to replace previous revealer notification
+        # Translators: Description, notification
+        toast = Adw.Toast.new(_("Loading"))
+        toast.set_timeout(20.0)
+        self._toast_overlay.add_toast(toast)
 
-        def callback():
+        def load(toast: Adw.Toast) -> None:
             if self.__note_manager.older_notes_model is None:
                 self.__load_older_notes_worker()
             else:
                 self._note_list.populate_older_notes(self.__note_manager.older_notes_model)
             self.__invalidate_view(check_older_notes=False)
             self._note_list.update_category_labels(self.__category)
-            if self._notification.get_reveal_child():
-                self._notification.hide()
+            toast.dismiss()
 
-        # Translators: Description, notification
-        self._notification.show(
-            _("Loading"),
-            immediate=animate_in_notification,
-            callback=lambda: GLib.idle_add(callback),
-        )
+        GLib.idle_add(load, toast)
 
     def __create_and_open_note(self, content: Optional[str] = None) -> None:
         note = self.__note_manager.create_note(self.__category)
@@ -665,6 +671,7 @@ class Index(Adw.BreakpointBin):
                 elif self._stack.get_visible_child() in (
                     self._empty,
                     self._first_start,
+                    self._loading,
                 ):
                     self._stack.set_visible_child(self._note_list_scroll)
 
@@ -680,14 +687,13 @@ class Index(Adw.BreakpointBin):
             self._search_header_bar.enter(False, not avoid_select_all)
             return
 
+        self.__close_sidebar()
+
         self._search_header_bar.enter(clear_text, False)
 
         all_category = self._sidebar.select_and_fetch_category(None, CategorySpecialPurpose.ALL)
         self.__category = all_category
 
-        if not iotas.config_manager.get_persist_sidebar() and not self._split_view.get_collapsed():
-            self._split_view.set_collapsed(True)
-
         self.__set_search_visible(True)
 
         if self.__note_manager.search_model is None:
@@ -696,8 +702,6 @@ class Index(Adw.BreakpointBin):
     def __init_search_model(self) -> None:
         model = self.__note_manager.initiate_search_model()
         self._note_list.populate_search(model)
-        if self._notification.get_reveal_child():
-            self._notification.hide()
 
     def __apply_search(self) -> None:
         if not self.__searching():
@@ -720,8 +724,7 @@ class Index(Adw.BreakpointBin):
             self._note_list.restrict_for_search_by_ids(None)
 
     def __start_selection(self) -> None:
-        if not iotas.config_manager.get_persist_sidebar() and not self._split_view.get_collapsed():
-            self._split_view.set_collapsed(True)
+        self.__close_sidebar()
         self.__searching_before_selection = self.__searching()
         self._header_stack.set_visible_child(self._selection_header_bar)
         # Prevent focus jumping to top row of listbox and resulting scroll
@@ -800,9 +803,8 @@ class Index(Adw.BreakpointBin):
         if loaded_older:
             self._note_list.populate_older_notes(self.__note_manager.older_notes_model)
 
-        self._stack.set_visible(True)
         self._note_list.refresh_section_visibility(self.__category)
-        self._notification.hide()
+        self._note_list.clear_selections()
 
         self.__initialised = True
         GLib.idle_add(self._note_list.update_category_labels, self.__category)
@@ -864,53 +866,54 @@ class Index(Adw.BreakpointBin):
                 )
                 # close_sidebar is False as we're closing it below
                 self.__on_category_activated(category, False)
-        if not iotas.config_manager.get_persist_sidebar() and not self._split_view.get_collapsed():
-            self.__close_sidebar()
-        elif self.__selecting():
-            self.__cancel_selection()
-        elif self.__searching():
-            self.__end_search()
+        closed_the_sidebar = self.__close_sidebar()
+        if not closed_the_sidebar:
+            if self.__selecting():
+                self.__cancel_selection()
+            elif self.__searching():
+                self.__end_search()
 
-    def __close_sidebar(self) -> None:
-        if self.get_width() < self.BREAKPOINT_WIDTH:
-            self._split_view.set_show_content(True)
-        else:
-            self._split_view.set_collapsed(True)
+    def __close_sidebar(self) -> bool:
+        if iotas.config_manager.get_pin_sidebar() and self.get_width() >= self.BREAKPOINT_WIDTH:
+            return False
+        elif not self._overlay_view.get_show_sidebar():
+            return False
+
+        self._overlay_view.set_show_sidebar(False)
         if self.get_root().using_keyboard_navigation:
             self._note_list.grab_focus()
         self.__update_header_buttons_for_state()
+        return True
 
     def __toggle_sidebar(self) -> None:
-        if iotas.config_manager.get_persist_sidebar() and self.get_width() >= self.BREAKPOINT_WIDTH:
-            # Persistent sidebar is open, let's just throw the focus around
+        if iotas.config_manager.get_pin_sidebar() and self.get_width() >= self.BREAKPOINT_WIDTH:
+            # Pinned sidebar is open, let's just throw the focus around
             if self._sidebar.has_focus_within():
                 self.get_root().using_keyboard_navigation = True
                 self._note_list.grab_focus()
             else:
                 self._sidebar.take_focus()
         else:
-            if self.get_width() < self.BREAKPOINT_WIDTH:
-                # Thin / within breakpoint window, toggle sidebar and content
-                self._split_view.set_show_content(not self._split_view.get_show_content())
+            if self._overlay_view.get_show_sidebar():
+                self._overlay_view.set_show_sidebar(False)
+                if self.get_root().using_keyboard_navigation:
+                    self._note_list.grab_focus()
             else:
-                # Wider / beyond breakpoint window
-                if self._split_view.get_collapsed():
-                    self._sidebar.take_focus()
-                    self._split_view.set_collapsed(False)
-                else:
-                    self._split_view.set_collapsed(True)
-                    if self.get_root().using_keyboard_navigation:
-                        self._note_list.grab_focus()
-            self.__update_header_buttons_for_state()
+                self._overlay_view.set_show_sidebar(True)
+                self._sidebar.take_focus()
 
-    def __update_for_sidebar_persistence_change(self, width: int = -1) -> None:
+    def __update_for_sidebar_pin_change(self, width: int = -1) -> None:
         if width == -1:
             width = self.get_width()
-        persist = iotas.config_manager.get_persist_sidebar()
-        if persist:
-            self._split_view.set_collapsed(width < self.BREAKPOINT_WIDTH)
+        if width >= self.BREAKPOINT_WIDTH:
+            pin = iotas.config_manager.get_pin_sidebar()
+            self._overlay_view.set_pin_sidebar(pin)
+            self._overlay_view.set_show_sidebar(pin)
+            self._overlay_view.set_collapsed(not pin)
         else:
-            self._split_view.set_collapsed(True)
+            self._overlay_view.set_pin_sidebar(False)
+            self._overlay_view.set_show_sidebar(False)
+            self._overlay_view.set_collapsed(True)
         self.__update_header_buttons_for_state(width)
 
     def __update_header_buttons_for_state(self, width: int = -1) -> None:
@@ -919,17 +922,15 @@ class Index(Adw.BreakpointBin):
             # Used during initialisation
             if width == 0:
                 width = self.get_property("width-request")
-        persist = iotas.config_manager.get_persist_sidebar()
-        if persist:
-            show_content_button = width < self.BREAKPOINT_WIDTH
-            show_sidebar_close = width < self.BREAKPOINT_WIDTH
+        if width >= self.BREAKPOINT_WIDTH and iotas.config_manager.get_pin_sidebar():
+            show_sidebar_close = False
+            show_content_sidebar_button = False
+            show_sidebar_menu = True
         else:
-            show_content_button = self._split_view.get_collapsed() and (
-                width >= self.BREAKPOINT_WIDTH or self._split_view.get_show_content()
-            )
-            show_sidebar_close = not show_content_button
-        show_sidebar_menu = persist and not self._split_view.get_collapsed()
-        self._content_sidebar_button.set_visible(show_content_button)
+            show_sidebar_close = True
+            show_content_sidebar_button = True
+            show_sidebar_menu = False
+        self._content_sidebar_button.set_visible(show_content_sidebar_button)
         self._sidebar.show_buttons(show_sidebar_close, show_sidebar_menu)
         self._menu_button.set_visible(not show_sidebar_menu)
 
